import { defineComponent, h, Transition, createSSRApp } from 'vue';
import { u as useLoadResult, R as RouterView, b as useHead, d as RouterLink, e as createHead } from './index-daa12a84.js';
import { NotFoundComponent, ErrorComponent } from 'dot-ream/templates/shared-exports.js';
import { callAsync } from 'dot-ream/templates/enhance-app.js';

const getProps = (props, keys) => {
  return keys.reduce((res, key) => {
    return {...res, [key]: props[key]};
  }, {});
};
const ReamPage = defineComponent({
  name: "ReamPage",
  inheritAttrs: false,
  setup() {
    return () => {
      const loadResult = useLoadResult();
      return h(RouterView, null, {
        default: (props) => {
          const {meta} = props.route;
          const transition = meta.transition === false ? {name: void 0} : typeof meta.transition === "string" ? {name: meta.transition} : meta.transition || {};
          const transitionProps = {
            name: "page",
            mode: "out-in",
            ...transition,
            onBeforeEnter() {
              _ream.event.emit("trigger-scroll");
              if (transition.onBeforeEnter) {
                transition.onBeforeEnter();
              }
            }
          };
          return h(Transition, transitionProps, () => {
            const componentProps = props.Component.type.props;
            return h(props.Component, {
              ...getProps(loadResult.props || {}, Array.isArray(componentProps) ? componentProps : componentProps ? Object.keys(componentProps) : [])
            });
          });
        }
      });
    };
  }
});

const createApp = async ({router, initialState}) => {
  const app = createSSRApp({
    setup() {
      useHead({
        meta: [
          {charset: "utf-8"},
          {name: "viewport", content: "width=device-width,initial-scale=1"}
        ]
      });
    },
    computed: {
      initialState() {
        return initialState;
      },
      loadResult() {
        const {load} = initialState;
        const routePath = this.$route.path;
        if (load[routePath]) {
          return load[routePath];
        }
        if (Object.keys(load).length === 1 && load["/404.html"]) {
          return load["/404.html"];
        }
        return {};
      }
    },
    render() {
      const {notFound, error} = this.loadResult;
      if (notFound) {
        return h(NotFoundComponent);
      }
      if (error) {
        return h(ErrorComponent);
      }
      return h(ReamPage);
    }
  });
  const head = createHead();
  app.use(router);
  app.use(head);
  app.component(ReamPage.name, ReamPage);
  app._context.components[RouterLink.name] = RouterLink;
  await callAsync("extendApp", {app, router, initialState});
  return {
    app,
    router
  };
};

export { createApp as c };
