'use strict';

var fs = require('fs');
var esbuild = require('esbuild');
var consola = require('consola');
var chalk = require('chalk');
var getViteConfig = require('./get-vite-config-60ced8ef.js');
var vite = require('vite');
require('path');
require('fs-extra');
require('./index.js');
require('http');
require('resolve-from');
require('@vitejs/plugin-vue');
require('@babel/core');
require('babel-plugin-eliminator');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);
var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);

async function build(api) {
  const serverConfig = getViteConfig.getViteConfig(api, true);
  const clientConfig = getViteConfig.getViteConfig(api, false);
  await vite.build(serverConfig);
  await vite.build(clientConfig);
}
async function buildStandalone(api) {
  const clientManifest = await fs__default['default'].promises.readFile(api.resolveDotReam("manifest/client-manifest.json"), "utf8");
  const ssrManifest = await fs__default['default'].promises.readFile(api.resolveDotReam("manifest/ssr-manifest.json"), "utf8");
  const exportManifest = await fs__default['default'].promises.readFile(api.resolveDotReam("manifest/export-manifest.json"), "utf8").catch(() => `{}`);
  const htmlTemplate = await fs__default['default'].promises.readFile(api.resolveDotReam("client/index.html"), "utf8");
  await fs__default['default'].promises.writeFile(api.resolveDotReam("server/standalone.js"), `
  import {createHandler as _createHandler} from 'ream/server'

  const serverEntry = require('./server-entry')

  export const createHandler = (cwd) => _createHandler({
    cwd,
    clientManifest: ${clientManifest},
    ssrManifest: ${ssrManifest},
    exportManifest: ${exportManifest},
    serverEntry,
    htmlTemplate: ${JSON.stringify(htmlTemplate)}
  })
  `);
  const inputFile = api.resolveDotReam("server/standalone.js");
  const outFile = api.resolveDotReam("server/standalone-bundle.js");
  const {warnings} = await esbuild.build({
    entryPoints: [inputFile],
    outfile: outFile,
    bundle: true,
    format: "cjs",
    platform: "node",
    minify: !process.env.DEBUG,
    logLevel: "error",
    target: "es2018"
  });
  for (const warning of warnings) {
    if (!warning.text.includes('Indirect calls to "require" will not be bundled')) {
      consola__default['default'].warn(chalk__default['default'].bold(warning.text));
      if (warning.location) {
        console.log(`${warning.location.file}:${warning.location.line}:${warning.location.column}`);
        console.log(warning.location.lineText);
      }
    }
  }
}

exports.build = build;
exports.buildStandalone = buildStandalone;
