'use strict';

var cac = require('cac');
var consola = require('consola');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);

var version = "5.0.0-beta.11";

const DEFAULT_PORT = 4e3;
const cli = cac.cac();
cli.command("[dir]", "Serve a directory in dev mode", {
  ignoreOptionDefaultValue: true
}).alias("dev").option("--host <host>", "Server host (default: localhost)").option("--port <port>", "Server port (default: 3000)").action(handleError(async (rootDir, options) => {
  const {Ream} = await Promise.resolve().then(function () { return require('./index.js'); });
  const app = new Ream({
    rootDir,
    dev: true,
    host: options.host,
    port: options.port || DEFAULT_PORT
  });
  await app.startServer();
}));
cli.command("build [dir]", "Build a directory for production", {
  ignoreOptionDefaultValue: true
}).option("--mode <mode>", "Set a custom mode for your app").option("--standalone", "Bundle external dependencies in server code").action(handleError(async (rootDir, flags) => {
  const {Ream} = await Promise.resolve().then(function () { return require('./index.js'); });
  const app = new Ream({
    rootDir,
    mode: flags.mode,
    dev: false
  });
  await app.build({standalone: flags.standalone});
}));
cli.command("export [dir]", "Export a hybrid site to a static site").option("--mode <mode>", "Set a custom mode for your app").action(handleError(async (rootDir, flags) => {
  const {Ream} = await Promise.resolve().then(function () { return require('./index.js'); });
  const app = new Ream({
    rootDir,
    mode: flags.mode,
    dev: false
  });
  await app.build({fullyExport: true}).catch(handleError);
}));
cli.command("start [dir]", "Start a production server").option("--host <host>", "Server host (default: localhost)").option("--port <port>", "Server port (default: 3000)").action(handleError(async (rootDir = ".", options) => {
  const {Ream} = await Promise.resolve().then(function () { return require('./index.js'); });
  const app = new Ream({
    rootDir,
    dev: false,
    host: options.host,
    port: options.port || DEFAULT_PORT
  });
  await app.startServer();
}));
cli.version(version);
cli.help();
cli.parse();
function handleError(fn) {
  return async (...args) => {
    try {
      await fn(...args);
    } catch (error) {
      consola__default['default'].error(error);
      process.exitCode = 1;
    }
  };
}
