'use strict';

var fs = require('fs');
var server = require('./server.js');
var index = require('./index.js');
require('path');
require('sirv');
require('querystring');
require('@egoist/router');
require('@vue/server-renderer');
require('serialize-javascript');
require('http');
require('vite');
require('resolve-from');
require('consola');
require('fs-extra');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

const SERVER_ENTRY_PATH = index.resolveOwnDir(`app/server-entry.js`);
const collectCssUrls = (mods, styles) => {
  for (const mod of mods) {
    if (mod.ssrModule && mod.file && mod.id) {
      if (mod.file.endsWith(".css") || /\?vue&type=style/.test(mod.id)) {
        styles.set(mod.url, mod.ssrModule.default);
      }
    }
    if (mod.importedModules.size > 0) {
      collectCssUrls(mod.importedModules, styles);
    }
  }
};
const getRequestHandler = async (api) => {
  const viteDevServer = api.viteDevServer;
  return async (req, res) => {
    await viteDevServer._pendingReload;
    const serverEntry = await viteDevServer.ssrLoadModule(`/@fs/${SERVER_ENTRY_PATH}`);
    const matchedMods = viteDevServer.moduleGraph.getModulesByFile(SERVER_ENTRY_PATH);
    const styles = new Map();
    if (matchedMods) {
      collectCssUrls(matchedMods, styles);
    }
    const inlineStyles = [...styles.values()].map((style) => `<style>${style}</style>`).join("\n");
    let htmlTemplate = await fs__default['default'].promises.readFile(api.resolveSrcDir("index.html"), "utf8");
    htmlTemplate = htmlTemplate.replace("<!--ream-head-->", `$&${inlineStyles}`);
    htmlTemplate = await viteDevServer.transformIndexHtml(req.url, htmlTemplate);
    const handler = await server.createHandler({
      cwd: api.rootDir,
      serverEntry,
      htmlTemplate,
      dev: true,
      before(server) {
        server.use(viteDevServer.middlewares);
      },
      fixStacktrace: (err) => viteDevServer.ssrFixStacktrace(err)
    });
    return handler(req, res);
  };
};

exports.getRequestHandler = getRequestHandler;
