'use strict';

var path = require('path');
var fs = require('fs-extra');
var consola = require('consola');
var chalk = require('chalk');
var serializeJavascript = require('serialize-javascript');
var server = require('./server.js');
var promiseQueue = require('@egoist/promise-queue');
var index = require('./index.js');
require('fs');
require('sirv');
require('querystring');
require('@egoist/router');
require('@vue/server-renderer');
require('http');
require('vite');
require('resolve-from');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);
var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
var serializeJavascript__default = /*#__PURE__*/_interopDefaultLegacy(serializeJavascript);

const flattenRoutes = async (routes, parentPath = "") => {
  const result = [];
  for (const route of routes) {
    const component = typeof route.component === "function" ? await route.component() : {};
    const path = [parentPath, route.path].filter(Boolean).join("/");
    const page = {
      load: component.$$load,
      preload: component.$$preload,
      getStaticPaths: component.$$getStaticPaths
    };
    if (route.children) {
      const childRoutes = await flattenRoutes(route.children, path);
      result.push(...childRoutes.map((r) => {
        r.matched.unshift(page);
        return r;
      }));
    } else {
      result.push({path, matched: [page]});
    }
  }
  return result;
};

function getHref(attrs) {
  const match = /href\s*=\s*(?:"(.*?)"|'(.*?)'|([^\s>]*))/.exec(attrs);
  return match && (match[1] || match[2] || match[3]);
}
const exportSite = async (dotReamDir, fullyExport) => {
  const exportDir = path__default['default'].join(dotReamDir, fullyExport ? "client" : "export");
  const serverAssets = await index.getServerAssets(dotReamDir);
  if (!fullyExport)
    return;
  const clientRoutes = await flattenRoutes(serverAssets.serverEntry.clientRoutes);
  const staticPaths = [];
  const exportManifest = {staticPages: []};
  await Promise.all(clientRoutes.map(async (route) => {
    const shouldExport = route.matched.every((m) => fullyExport || !m.load);
    if (!shouldExport) {
      return;
    }
    if (route.path === "/:404(.*)") {
      staticPaths.push("/404.html");
    } else {
      const isDynamicPath = route.path.includes(":");
      let fallback = false;
      const isStatic = route.matched.every((m) => !m.preload);
      if (!isStatic)
        return;
      if (isDynamicPath) {
        const paths = [];
        for (const m of route.matched) {
          if (m.getStaticPaths) {
            const res = await m.getStaticPaths();
            paths.push(...res.paths);
            if (res.fallback != null) {
              fallback = res.fallback;
            }
          }
        }
        if (paths.length === 0) {
          consola__default['default'].warn(`No static paths provided for ${route.path}, skipped exporting`);
        }
        for (const path2 of paths) {
          const staticPath = route.path.replace("(.*)", "").replace(/:([^\/]+)/g, (_, p1) => path2.params[p1]);
          staticPaths.push(staticPath);
        }
      } else {
        staticPaths.push(route.path);
      }
      exportManifest.staticPages.push({path: route.path, fallback});
    }
  }));
  await fs__default['default'].outputFile(path__default['default'].join(dotReamDir, "manifest/export-manifest.json"), JSON.stringify(exportManifest), "utf8");
  const queue = new promiseQueue.PromiseQueue(async (jobId, url) => {
    consola__default['default'].info(chalk__default['default'].dim(jobId));
    const router = await server.createClientRouter(serverAssets.serverEntry, url);
    const route = router.currentRoute.value;
    const {html = "", loadResult} = await server.render({
      url: route.path,
      ...serverAssets,
      router,
      loadOptions: {
        params: route.params,
        host: "localhost",
        headers: {},
        query: {},
        path: route.path
      }
    });
    const htmlPath = server.getExportOutputPath(route.path, "html", exportDir);
    const jsonPath = server.getExportOutputPath(route.path, "json", exportDir);
    await Promise.all([
      fs__default['default'].outputFile(htmlPath, html, "utf8"),
      (loadResult.hasPreload || loadResult.hasLoad) && await fs__default['default'].outputFile(jsonPath, serializeJavascript__default['default'](loadResult, {isJSON: true}), "utf8")
    ]);
    if (fullyExport) {
      let match = null;
      const LINK_RE = /<a ([\s\S]+?)>/gm;
      while (match = LINK_RE.exec(html)) {
        const href = getHref(match[1]);
        if (href) {
          const url2 = new URL(href, "http://self");
          if (url2.host === "self") {
            queue.add(`Exporting ${url2.pathname}`, url2.pathname);
          }
        }
      }
    }
  }, {maxConcurrent: 100});
  for (const path2 of staticPaths) {
    queue.add(`Exporting ${path2}`, path2);
  }
  await queue.run();
  if (fullyExport) {
    consola__default['default'].success(`Your app has been exported to ${path__default['default'].relative(process.cwd(), path__default['default'].join(dotReamDir, "client"))}`);
  }
};

exports.exportSite = exportSite;
