'use strict';

var path = require('path');
var fs = require('fs-extra');
var index = require('./index.js');
var vuePlugin = require('@vitejs/plugin-vue');
var core = require('@babel/core');
var eliminator = require('babel-plugin-eliminator');
require('http');
require('vite');
require('resolve-from');
require('consola');
require('fs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var vuePlugin__default = /*#__PURE__*/_interopDefaultLegacy(vuePlugin);
var eliminator__default = /*#__PURE__*/_interopDefaultLegacy(eliminator);

const babelPlugin = () => {
  let needSourceMap = false;
  return {
    name: `ream:babel`,
    configResolved(config) {
      needSourceMap = Boolean(config.build.sourcemap);
    },
    transform(code, id, ssr) {
      if (ssr || id.includes("node_modules"))
        return;
      if (!/\.[jt]sx$/.test(id) && !/\.vue$/.test(id) && !id.endsWith("?vue&type=script&lang.ts")) {
        return;
      }
      const result = core.transformSync(code, {
        sourceFileName: id,
        sourceMaps: needSourceMap,
        plugins: [
          [
            eliminator__default['default'],
            {
              namedExports: ["load", "preload", "getStaticPaths"],
              done(state) {
                if (state.removedNamedExports.has("load") && state.removedNamedExports.has("preload")) {
                  throw new Error(`You can't use "load" and "preload" in the same page.`);
                }
              }
            }
          ]
        ]
      });
      return result && {
        code: result.code.replace(`import.meta.hot.accept(({
  default: updated,
  _rerender_only
}) => {`, `
$&
`),
        map: result.map
      };
    }
  };
};

const reamForceServerUpdatePlugin = (api) => {
  return {
    name: `ream:force-server-update`,
    handleHotUpdate(ctx) {
      const {moduleGraph} = api.viteDevServer;
      for (const [key, value] of moduleGraph.fileToModulesMap.entries()) {
        if (!key.includes("node_modules") && !key.includes(".ream")) {
          for (const mod of value) {
            moduleGraph.invalidateModule(mod);
          }
        }
      }
    }
  };
};
const moveManifestPlugin = (manifestDir) => {
  return {
    name: "ream:move-manifest",
    async writeBundle(options, bundle) {
      const files = [];
      for (const file in bundle) {
        const value = bundle[file];
        if (file === "manifest.json" || file === "ssr-manifest.json" && value.type === "asset") {
          files.push({
            name: file === "manifest.json" ? "client-manifest.json" : file,
            originalPath: path__default['default'].join(options.dir, file),
            content: value.source
          });
          delete bundle[file];
        }
      }
      if (files.length > 0) {
        await Promise.all(files.map(async (file) => {
          await fs__default['default'].remove(file.originalPath);
          await fs__default['default'].outputFile(path__default['default'].join(manifestDir, file.name), file.content, "utf8");
        }));
      }
    }
  };
};
const getViteConfig = (api, server) => {
  var _a;
  const ssrManifest = !server && !api.isDev;
  const entry = api.isDev ? void 0 : server ? "ream/dist/app/server-entry.js" : {client: "./index.html"};
  const config = {
    mode: api.mode,
    root: api.rootDir,
    configFile: false,
    plugins: [
      reamForceServerUpdatePlugin(api),
      vuePlugin__default['default']({
        include: [/\.vue$/]
      }),
      babelPlugin(),
      moveManifestPlugin(api.resolveDotReam("manifest")),
      {
        name: "inject-script-tag",
        transformIndexHtml: {
          enforce: "pre",
          transform() {
            return [
              {
                injectTo: "body",
                tag: "script",
                attrs: {
                  type: "module",
                  src: `/@fs/${index.resolveOwnDir("app/client-entry.js")}`
                }
              }
            ];
          }
        }
      }
    ],
    define: api.constants,
    resolve: {
      alias: {
        "@": api.resolveSrcDir(),
        "dot-ream": api.resolveDotReam()
      }
    },
    optimizeDeps: {
      exclude: ["ream/app", "@ream/fetch", "node-fetch"],
      include: ["vue"]
    },
    ssr: {
      external: [
        "vue",
        "vue/dist/vue.esm-bundler.js",
        "vue/dist/vue.runtime.esm-bundler.js"
      ],
      noExternal: ["ream/app", "ream"]
    },
    server: {
      middlewareMode: true,
      hmr: {
        port: ((_a = api.config.hmr) == null ? void 0 : _a.port) || api.port,
        server: api._devServer
      }
    },
    build: {
      minify: !api.isDev && !process.env.DEBUG,
      outDir: server ? api.resolveDotReam("server") : api.resolveDotReam("client"),
      ssrManifest,
      ssr: server,
      manifest: !api.isDev && !server,
      rollupOptions: entry ? {
        input: entry
      } : {}
    }
  };
  if (api.config.vite) {
    api.config.vite(config, {dev: api.isDev});
  }
  return config;
};

exports.getViteConfig = getViteConfig;
