import http__default from 'http';
import { UserConfig, ViteDevServer } from 'vite';
import { ServerEntry } from './server';
import '../load-1f537ab8';
import 'vue';
import 'querystring';
import '@egoist/router';

/** Mark some properties as required, leaving others unchanged */
declare type MarkRequired<T, RK extends keyof T> = Exclude<T, RK> & Required<Pick<T, RK>>;

declare type OnFileChangeCallback = (this: Ream, event: 'add' | 'addDir' | 'change' | 'unlink' | 'unlinkDir', filepath: string) => any;
declare type ReamPlugin = {
    name: string;
    /**
     * Before start dev server / bundling
     * Never call before starting production server
     */
    prepare?: (this: Ream) => void | Promise<void>;
    enhanceAppFiles?: (this: Ream) => string[];
    enhanceServerFiles?: (this: Ream) => string[];
    onFileChange?: OnFileChangeCallback;
};

declare type State = {
    constants: {
        [k: string]: string;
    };
};

interface Options {
    rootDir?: string;
    srcDir?: string;
    dev?: boolean;
    mode?: string;
    host?: string;
    port?: number;
}
declare type Route = {
    name?: string;
    path: string;
    file: string;
    isEndpoint: boolean;
    children?: Route[];
};
interface Endpoint extends Route {
    isEndpoint: true;
    children: undefined;
}
declare type ReamConfig = {
    env?: Record<string, string>;
    plugins?: Array<ReamPlugin>;
    imports?: string[];
    vite?: (viteConfig: UserConfig, opts: {
        dev: boolean;
    }) => void;
    pages?: (defaultPages: Route[]) => Promise<Route[]> | Route[];
    endpoints?: (defaultEndpoints: Endpoint[]) => Promise<Endpoint[]> | Endpoint[];
    hmr?: {
        /**
         * Configure the port that hmr client connects to
         *
         * Note that this doesn't affect which port the server listens to,
         * the actual server port is alway the value of `--port`
         *
         * You might set this to `443` if you're running Ream dev server on repl.it
         * or other cloud IDEs.
         */
        port?: number;
    };
};
declare const defineReamConfig: (config: ReamConfig) => ReamConfig;
declare const OWN_DIR: string;
declare const resolveOwnDir: (...args: string[]) => string;
declare class Ream {
    state: State;
    rootDir: string;
    srcDir: string;
    isDev: boolean;
    inlineConfig: ReamConfig;
    config: MarkRequired<ReamConfig, 'env' | 'plugins' | 'imports'>;
    configPath?: string;
    viteDevServer?: ViteDevServer;
    userEnv: Record<string, string>;
    mode: string;
    host?: string;
    port?: number;
    _devServer?: http__default.Server;
    constructor(options: Options, inlineConfig?: ReamConfig);
    resolveRootDir(...args: string[]): string;
    resolveSrcDir(...args: string[]): string;
    resolveDotReam(...args: string[]): string;
    resolveInPackage(pkg: string, target: string): string;
    loadConfig(): Promise<void>;
    get env(): Record<string, string>;
    get constants(): Record<string, string>;
    ensureEnv(name: string, defaultValue?: string): string | undefined;
    prepare(type: 'build' | 'dev' | 'start'): Promise<void>;
    localResolve(name: string): string | undefined;
    localRequire(name: string): any;
    getRequestHandler(): Promise<(_req: http__default.IncomingMessage, _res: http__default.ServerResponse) => void>;
    startServer(): Promise<void>;
    build({ fullyExport, standalone, }: {
        fullyExport?: boolean;
        standalone?: boolean;
    }): Promise<void>;
}

declare const getServerAssets: (dotReamDir: string) => Promise<{
    serverEntry: ServerEntry;
    ssrManifest: any;
    clientManifest: any;
    exportManifest: Record<string, any> | undefined;
    htmlTemplate: string;
}>;

export { Endpoint, OWN_DIR, OnFileChangeCallback, Options, Ream, ReamConfig, ReamPlugin, Route, defineReamConfig, getServerAssets, resolveOwnDir };
