import * as http from 'http';
import { IncomingMessage, ServerResponse } from 'http';
import { m as LoadOptions, R as Router$1, L as LoadResultNormalized, c as Load, H as HTMLResult, n as RouteRecordRaw } from '../load-1f537ab8';
export { G as GetStaticPaths, s as GetStaticPathsResult, c as Load, m as LoadOptions, o as LoadResult, L as LoadResultNormalized, P as Preload, p as PreloadOptions, q as PreloadResult } from '../load-1f537ab8';
import { ParsedUrlQuery } from 'querystring';
import { Router, HTTPMethod } from '@egoist/router';
import 'vue';

declare type SendData = NodeJS.ReadableStream | object | string | Buffer;

declare type SimpleHandleFunction<TReq = any, TRes = any> = (req: TReq, res: TRes) => void | Promise<void>;
declare type NextHandleFunction<TReq = any, TRes = any> = (req: TReq, res: TRes, next: NextFunction) => void | Promise<void>;
declare type HandleFunction<TReq = any, TRes = any> = SimpleHandleFunction<TReq, TRes> | NextHandleFunction<TReq, TRes>;
declare type NextFunction = (err?: ConnectError) => void;
interface ConnectRequest extends IncomingMessage {
    originalUrl: string;
    path: string;
    query: ParsedUrlQuery;
    params: Record<string, string | string[]>;
}
interface ConnectResponse extends ServerResponse {
}
declare type OnError<TReq, TRes> = (error: ConnectError | string, req: TReq, res: TRes) => void;
declare class ConnectError extends Error {
    code?: number;
    status?: number;
}
declare type Options$2<TReq, TRes> = {
    onError?: OnError<TReq, TRes>;
    onNoMatch?: SimpleHandleFunction<TReq, TRes>;
};
declare class Connect<TReq extends ConnectRequest, TRes extends ConnectResponse> {
    onError: OnError<TReq, TRes>;
    onNoMatch: SimpleHandleFunction<TReq, TRes>;
    router: Router<HandleFunction<TReq, TRes>>;
    constructor(options?: Options$2<TReq, TRes>);
    add(method: HTTPMethod, route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]): this;
    use(route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]): this;
    use(handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]): this;
    get: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    head: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    patch: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    options: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    connect: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    delete: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    trace: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    post: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    put: (route: string, handler: HandleFunction<TReq, TRes>, ...handlers: HandleFunction<TReq, TRes>[]) => this;
    handler: (_req: IncomingMessage, _res: ServerResponse) => void;
}

declare type ReamServerHandler = (req: ReamServerRequest, res: ReamServerResponse, next: NextFunction) => void | Promise<void>;
interface ReamServerRequest extends ConnectRequest {
    url: string;
    path: string;
    params: {
        [k: string]: any;
    };
    isLoadRequest?: boolean;
}
interface ReamServerResponse extends ConnectResponse {
    send: (data: SendData) => void;
}
declare type ReamServer = Connect<ReamServerRequest, ReamServerResponse>;

declare function render({ url, loadOptions, ssrManifest, serverEntry, isLoadRequest, clientManifest, router, notFound, htmlTemplate, }: {
    url: string;
    loadOptions: LoadOptions;
    ssrManifest?: any;
    serverEntry: ServerEntry;
    isLoadRequest?: boolean;
    clientManifest?: any;
    router: Router$1;
    /** Render the 404 page */
    notFound?: boolean;
    htmlTemplate: string;
}): Promise<{
    html?: string;
    loadResult: LoadResultNormalized;
}>;
declare function renderToHTML(options: {
    url: string;
    path: string;
    loadResult: LoadResultNormalized;
    router: Router$1;
    serverEntry: ServerEntry;
    ssrManifest?: any;
    clientManifest?: any;
    htmlTemplate: string;
}): Promise<string>;
declare function loadPageData(globalLoad: Load | undefined, components: any[], options: LoadOptions): Promise<LoadResultNormalized>;

declare type Options$1 = {
    max?: number;
    ttl?: number;
};
interface CacheItem<ValueType> {
    key: string;
    expiry: number;
    ttl: number;
    prev: CacheItem<ValueType> | null;
    next: CacheItem<ValueType> | null;
    value: ValueType;
}
declare class LRU<ValueType> {
    first: CacheItem<ValueType> | null;
    last: CacheItem<ValueType> | null;
    items: {
        [k: string]: CacheItem<ValueType>;
    };
    private max;
    size: number;
    private ttl;
    constructor(options?: Options$1);
    has(key: string): boolean;
    clear(): void;
    delete(key: string): void;
    evict(): void;
    get(key: string): ValueType | undefined;
    keys(): string[];
    set(key: string, value: ValueType, ttl?: number, bypassChecking?: boolean): void;
}

declare type PageCache = {
    html?: string;
    loadResult: LoadResultNormalized;
    isStale?: boolean;
};
declare type Options = {
    exportDir: string;
    flushToDisk: boolean;
    exportManifest?: ExportManifest;
    /**
     * Make `.get` always return `undefined`
     * a.k.a use as a write only storage
     * Mainly used for `ream export`
     */
    writeOnly?: boolean;
};
declare const getExportOutputPath: (pathname: string, type: 'html' | 'json', exportDir: string) => string;
declare class ExportCache {
    cache: LRU<PageCache>;
    options: Options;
    constructor(options: Options);
    getOutputPath(pathname: string, type: 'html' | 'json'): string;
    get writeOnly(): boolean | undefined;
    findStaticPage(rawPath: string): {
        path: string;
        fallback?: boolean | undefined;
    } | undefined;
    get(pathname: string): Promise<PageCache | undefined>;
    set(pathname: string, { html, loadResult }: {
        html?: string;
        loadResult: LoadResultNormalized;
    }, flushToDisk?: boolean): Promise<void>;
}

/**
 * Hook: `extendServer`
 */
declare type ExtendServer = (context: {
    server: Connect<ReamServerRequest, ReamServerResponse>;
}) => void | Promise<void>;
declare type GetInitialHTMLContext = {
    head: string;
    main: string;
    scripts: string;
    htmlAttrs: string;
    bodyAttrs: string;
};
/**
 * Hook: `getInitialHTML`
 */
declare type GetInitialHTML = (context: GetInitialHTMLContext) => string | undefined | Promise<string | undefined>;

declare type ServerEntry = {
    render: any;
    createClientRouter: () => Router$1;
    createServerRouter: () => Router$1;
    renderHead: (app: any) => HTMLResult;
    ErrorComponent: any;
    serverRoutes: RouteRecordRaw[];
    clientRoutes: RouteRecordRaw[];
    enhanceApp: {
        callAsync: (name: string, context: any) => Promise<void>;
    };
    enhanceServer: {
        hasExport: (name: string) => boolean;
        getInitialHTML: GetInitialHTML;
        callAsync: (name: string, context: any) => Promise<void>;
    };
};
declare type ExportManifest = {
    staticPages: {
        path: string;
        fallback?: boolean;
    }[];
};
declare type CreateServerOptions = {
    /**
     * Development mode
     * @default {false}
     */
    dev?: boolean;
    /**
     * Path to your project
     */
    cwd?: string;
    before?: (server: ReamServer) => void;
    fixStacktrace?: (err: Error) => void;
    serverEntry: ServerEntry;
    exportManifest?: any;
    ssrManifest?: any;
    clientManifest?: any;
    htmlTemplate: string;
};

declare const createClientRouter: (serverEntry: ServerEntry, url: string) => Promise<Router$1>;
declare const getLoadOptions: (req: ReamServerRequest) => LoadOptions;
declare function createHandler(options: CreateServerOptions): Promise<(_req: http.IncomingMessage, _res: http.ServerResponse) => void>;

export { Connect, ExportCache, ExportManifest, ExtendServer, GetInitialHTML, ReamServerHandler, ReamServerRequest, ReamServerResponse, ServerEntry, createClientRouter, createHandler, getExportOutputPath, getLoadOptions, loadPageData, render, renderToHTML };
